package jp.co.veritrans.vtweb.mdk.conf;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

/**
 * VTWEBMDK.propertiesの値を保持するクラス
 */
public final class MerchantConf {

    /** 自インスタンス */
    private static MerchantConf info = null;

    /** ログ出力用インスタンス */
    private static Logger log = Logger.getLogger(MerchantConf.class);

    /** マーチャントID */
    private String merchantID = "";

    /** カード売上フラグ */
    private String cardCaptureFlag = "";

    /** シード */
    private String seed = "";

    /** 決済方法 */
    private String settlementType = "";

    /** VT Web 暗号鍵取得URL */
    private String vtWebRegistUrl = "";

    /** VT Web 決済URL */
    private String vtWebSettlementUrl = "";

    /** 決済完了後戻りURL */
    private String finishPaymentReturnUrl = "";

    /** 未決済時戻りURL */
    private String unfinishPaymentReturnUrl = "";

    /** 決済エラー時戻りURL */
    private String errorPaymentReturnUrl = "";

    /** 決済結果通知先URL */
    private String finishPaymentAccessUrl = "";

    /** 言語選択可否フラグ */
    private String langEnableFlag = "";

    /** 使用言語 */
    private String lang = "";

    /** ダミー決済フラグ */
    private String dummyPaymentFlag = "";

    /** モバイルEdy用ショップ名 */
    private String shopName = "";

    /** Suica用商品名 */
    private String screenTitle = "";

    /** 請求内容 */
    private String contents = "";

    /** 請求内容カナ */
    private String contentsKana = "";

    /** VT Webに送信するダミー商品ID */
    private String dummyCommodityId = "";

    /** VT Webに送信するダミーJAN_CODE */
    private String dummyCommodityJanCode = "";

    /**
     * コンストラクタ。外部からの生成を防止
     *
     */
    private MerchantConf() {
        ResourceBundle bdl = null;
        try {
            bdl = ResourceBundle.getBundle("VTWEBMDK", Locale.JAPAN);
            merchantID = bdl.getString("MERCHANT_ID");
            seed = bdl.getString("SEED");
            settlementType = bdl.getString("SETTLEMENT_TYPE");
            vtWebRegistUrl = bdl.getString("VTWEB_REGIST_URL");
            vtWebSettlementUrl = bdl.getString("VTWEB_SETTLEMENT_URL");
            finishPaymentReturnUrl = bdl.getString("FINISH_PAYMENT_RETURN_URL");
            unfinishPaymentReturnUrl = bdl.getString("UNFINISH_PAYMENT_RETURN_URL");
            errorPaymentReturnUrl = bdl.getString("ERROR_PAYMENT_RETURN_URL");
            finishPaymentAccessUrl = bdl.getString("FINISH_PAYMENT_ACCESS_URL");
            langEnableFlag = bdl.getString("LANG_ENABLE_FLAG");
            lang = bdl.getString("LANG");
            dummyPaymentFlag = bdl.getString("DUMMY_PAYMENT_FLAG");
            cardCaptureFlag = bdl.getString("CARD_CAPTURE_FLAG");
            shopName = bdl.getString("SHOP_NAME");
            screenTitle = bdl.getString("SCREEN_TITLE");
            contents = bdl.getString("CONTENTS");
            contentsKana = bdl.getString("CONTENTS_KANA");
            dummyCommodityId = bdl.getString("DUMMY_COMMODITY_ID");
            dummyCommodityJanCode = bdl.getString("DUMMY_COMMODITY_JANCODE");
        } catch (MissingResourceException e) {
            log.fatal("VTWEBMDK.propetiesが読み込めません", e);
            return;
        }
    }

    /**
     * インスタンスを取得
     *
     * @return MerchantConfクラス
     */
    public static MerchantConf getInfo() {
        if (info == null) {
            info = new MerchantConf();
        }
        return info;
    }

    /**
     * 対象項目を数値化する。
     *
     * @param bdl Resource Bundleインスタンス
     * @param palName 対象項目のパラメータ名
     * @return 数値化した項目
     */
    private int getIntColumn(ResourceBundle bdl, String palName) {
        int retVal = 0;
        String payLimitStr = bdl.getString(palName);
        try {
            retVal = Integer.parseInt(payLimitStr);
        } catch (Exception nfe) {
            log.warn("数字ではありません。" + palName + " = " + payLimitStr, nfe);
        }
        return retVal;
    }

    /**
     * merchantIDの取得
     *
     * @return the merchantID of String
     */
    public String getMerchantID() {
        return merchantID;
    }

    /**
     * merchantIDの設定
     *
     * @param merchantID the merchantID to set
     */
    public void setMerchantID(String merchantID) {
        this.merchantID = merchantID;
    }

    /**
     * cardCaptureFlagの取得
     *
     * @return the cardCaptureFlag of String
     */
    public String getCardCaptureFlag() {
        return cardCaptureFlag;
    }

    /**
     * cardCaptureFlagの設定
     *
     * @param cardCaptureFlag the cardCaptureFlag to set
     */
    public void setCardCaptureFlag(String cardCaptureFlag) {
        this.cardCaptureFlag = cardCaptureFlag;
    }

    /**
     * seedの取得
     *
     * @return the seed of String
     */
    public String getSeed() {
        return seed;
    }

    /**
     * seedの設定
     *
     * @param seed the seed to set
     */
    public void setSeed(String seed) {
        this.seed = seed;
    }

    /**
     * settlementTypeの取得
     *
     * @return the settlementType of String
     */
    public String getSettlementType() {
        return settlementType;
    }

    /**
     * settlementTypeの設定
     *
     * @param settlementType the settlementType to set
     */
    public void setSettlementType(String settlementType) {
        this.settlementType = settlementType;
    }

    /**
     * vtWebRegistUrlの取得
     *
     * @return the vtWebRegistUrl of String
     */
    public String getVtWebRegistUrl() {
        return vtWebRegistUrl;
    }

    /**
     * vtWebRegistUrlの設定
     *
     * @param vtWebRegistUrl the vtWebRegistUrl to set
     */
    public void setVtWebRegistUrl(String vtWebRegistUrl) {
        this.vtWebRegistUrl = vtWebRegistUrl;
    }

    /**
     * vtWebSettlementUrlの取得
     *
     * @return the vtWebSettlementUrl of String
     */
    public String getVtWebSettlementUrl() {
        return vtWebSettlementUrl;
    }

    /**
     * vtWebSettlementUrlの設定
     *
     * @param vtWebSettlementUrl the vtWebSettlementUrl to set
     */
    public void setVtWebSettlementUrl(String vtWebSettlementUrl) {
        this.vtWebSettlementUrl = vtWebSettlementUrl;
    }

    /**
     * finishPaymentReturnUrlの取得
     *
     * @return the finishPaymentReturnUrl of String
     */
    public String getFinishPaymentReturnUrl() {
        return finishPaymentReturnUrl;
    }

    /**
     * finishPaymentReturnUrlの設定
     *
     * @param finishPaymentReturnUrl the finishPaymentReturnUrl to set
     */
    public void setFinishPaymentReturnUrl(String finishPaymentReturnUrl) {
        this.finishPaymentReturnUrl = finishPaymentReturnUrl;
    }

    /**
     * unfinishPaymentReturnUrlの取得
     *
     * @return the unfinishPaymentReturnUrl of String
     */
    public String getUnfinishPaymentReturnUrl() {
        return unfinishPaymentReturnUrl;
    }

    /**
     * unfinishPaymentReturnUrlの設定
     *
     * @param unfinishPaymentReturnUrl the unfinishPaymentReturnUrl to set
     */
    public void setUnfinishPaymentReturnUrl(String unfinishPaymentReturnUrl) {
        this.unfinishPaymentReturnUrl = unfinishPaymentReturnUrl;
    }

    /**
     * errorPaymentReturnUrlの取得
     *
     * @return the errorPaymentReturnUrl of String
     */
    public String getErrorPaymentReturnUrl() {
        return errorPaymentReturnUrl;
    }

    /**
     * errorPaymentReturnUrlの設定
     *
     * @param errorPaymentReturnUrl the errorPaymentReturnUrl to set
     */
    public void setErrorPaymentReturnUrl(String errorPaymentReturnUrl) {
        this.errorPaymentReturnUrl = errorPaymentReturnUrl;
    }

    /**
     * finishPaymentAccessUrlの取得
     *
     * @return the finishPaymentAccessUrl of String
     */
    public String getFinishPaymentAccessUrl() {
        return finishPaymentAccessUrl;
    }

    /**
     * finishPaymentAccessUrlの設定
     *
     * @param finishPaymentAccessUrl the finishPaymentAccessUrl to set
     */
    public void setFinishPaymentAccessUrl(String finishPaymentAccessUrl) {
        this.finishPaymentAccessUrl = finishPaymentAccessUrl;
    }

    /**
     * langEnableFlagの取得
     *
     * @return the langEnableFlag of String
     */
    public String getLangEnableFlag() {
        return langEnableFlag;
    }

    /**
     * langEnableFlagの設定
     *
     * @param langEnableFlag the langEnableFlag to set
     */
    public void setLangEnableFlag(String langEnableFlag) {
        this.langEnableFlag = langEnableFlag;
    }

    /**
     * langの取得
     *
     * @return the lang of String
     */
    public String getLang() {
        return lang;
    }

    /**
     * langの設定
     *
     * @param lang the lang to set
     */
    public void setLang(String lang) {
        this.lang = lang;
    }

    /**
     * dummyPaymentFlagの取得
     *
     * @return the dummyPaymentFlag of String
     */
    public String getDummyPaymentFlag() {
        return dummyPaymentFlag;
    }

    /**
     * dummyPaymentFlagの設定
     *
     * @param dummyPaymentFlag the dummyPaymentFlag to set
     */
    public void setDummyPaymentFlag(String dummyPaymentFlag) {
        this.dummyPaymentFlag = dummyPaymentFlag;
    }

    /**
     * shopNameの取得
     *
     * @return the shopName of String
     */
    public String getShopName() {
        return shopName;
    }

    /**
     * shopNameの設定
     *
     * @param shopName the shopName to set
     */
    public void setShopName(String shopName) {
        this.shopName = shopName;
    }

    /**
     * screenTitleの取得
     *
     * @return the screenTitle of String
     */
    public String getScreenTitle() {
        return screenTitle;
    }

    /**
     * screenTitleの設定
     *
     * @param screenTitle the screenTitle to set
     */
    public void setScreenTitle(String screenTitle) {
        this.screenTitle = screenTitle;
    }

    /**
     * contentsの取得
     *
     * @return the contents of String
     */
    public String getContents() {
        return contents;
    }

    /**
     * contentsの設定
     *
     * @param contents the contents to set
     */
    public void setContents(String contents) {
        this.contents = contents;
    }

    /**
     * contentsKanaの取得
     *
     * @return the contentsKana of String
     */
    public String getContentsKana() {
        return contentsKana;
    }

    /**
     * contentsKanaの設定
     *
     * @param contentsKana the contentsKana to set
     */
    public void setContentsKana(String contentsKana) {
        this.contentsKana = contentsKana;
    }

    /**
     * dummyCommodityIdの取得
     * @return the dummyCommodityId of String
     */
    public String getDummyCommodityId() {
        return dummyCommodityId;
    }

    /**
     * dummyCommodityIdの設定
     * @param dummyCommodityId the dummyCommodityId to set
     */
    public void setDummyCommodityId(String dummyCommodityId) {
        this.dummyCommodityId = dummyCommodityId;
    }

    /**
     * dummyCommodityJanCodeの取得
     * @return the dummyCommodityJanCode of String
     */
    public String getDummyCommodityJanCode() {
        return dummyCommodityJanCode;
    }

    /**
     * dummyCommodityJanCodeの設定
     * @param dummyCommodityJanCode the dummyCommodityJanCode to set
     */
    public void setDummyCommodityJanCode(String dummyCommodityJanCode) {
        this.dummyCommodityJanCode = dummyCommodityJanCode;
    }
}
