package jp.co.veritrans.vtweb.sample.server;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.co.veritrans.vtweb.mdk.bean.PostDataBox;
import jp.co.veritrans.vtweb.mdk.bean.ResultDataBean;
import jp.co.veritrans.vtweb.mdk.common.Constants;
import jp.co.veritrans.vtweb.mdk.common.ResultCheck;
import jp.co.veritrans.vtweb.mdk.common.StringUtil;

import org.apache.log4j.Logger;

/**
 * VTWebからの決済結果を受け取るServlet
 */
public class DoPostActionVTW extends HttpServlet {

    /**
     * serial version UID.
     */
    private static final long serialVersionUID = 8664728883149660895L;

    /**
     * Content-Type
     */
    private static String CONTENT_TYPE = "text/html; charset=" + Constants.VTW_ENCODE;

    /**
     * 個人情報項目
     */
    private static String[] PRIVATEINFO = { Constants.VTWRES_USER_NAME, Constants.VTWRES_USER_NAME_KANA, Constants.VTWRES_TELEPHONE_NO};

    /**
     * ログ出力用インスタンス
     */
    private Logger log = Logger.getLogger(DoPostActionVTW.class);

    /**
     * POSTされたリクエストを受け取る
     *
     * @param request リクエスト
     * @param response レスポンス
     * @throws IOException 入出力例外
     * @throws ServletException サーブレットの例外
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {

        log.info("決済結果確認開始");
        request.setCharacterEncoding(Constants.VTW_ENCODE);

        // リクエストの一覧をログ出力
        if (log.isInfoEnabled()) {
            Enumeration enu = request.getParameterNames();
            while (enu.hasMoreElements()) {
                String param = (String) enu.nextElement();
                String value = URLDecoder.decode(request.getParameter(param), Constants.VTW_ENCODE);
                log.info(param + ":" + value);
            }
        }

        PostDataBox dataBox = PostDataBox.getInstance();
        String orderId = URLDecoder.decode(request.getParameter(Constants.VTWRES_ORDER_ID), Constants.VTW_ENCODE);
        Map dataMap = dataBox.getBox(orderId);

        if (dataMap == null) {
            // 該当取引IDがないため、新しく登録
            dataMap = new HashMap();
            dataBox.setBox(orderId, dataMap);
        } else if (dataMap.containsKey(Constants.VTWRES_M_STATUS)) {
            StringBuffer sb = new StringBuffer();
            Iterator ite = dataMap.keySet().iterator();
            String entry = null;
            while (ite.hasNext()) {
                entry = (String)ite.next();
                sb.append(entry);
                sb.append("=");
                sb.append((String) dataMap.get(entry));
            }
            log.warn("該当取引ID    は受信済み。" + sb.toString());
            response.setContentType(CONTENT_TYPE);
            ServletOutputStream out = response.getOutputStream();

            out.println("ERR");
            return;
        }

        ResultDataBean databean = new ResultDataBean();

        // リクエストをマップに格納
        databean.setOrderId(orderId);
        databean.setmStatus(URLDecoder.decode(request.getParameter(Constants.VTWRES_M_STATUS), Constants.VTW_ENCODE));
        databean.setmErrMsg(URLDecoder.decode(request.getParameter(Constants.VTWRES_M_ERR_MSG), Constants.VTW_ENCODE));
        databean.setvResultCode(URLDecoder.decode(request.getParameter(Constants.VTWRES_V_RESULT_CODE), Constants.VTW_ENCODE));

        ResultCheck rcheck = new ResultCheck();
        boolean checkflag;

        // 内容を確認
        checkflag = rcheck.resultCheck(databean);

        // VTWへ返信
        if (checkflag) {
            log.info("内容確認成功");
            dataMap.put(Constants.VTWRES_M_STATUS, databean.getmStatus());
            dataMap.put(Constants.VTWRES_M_ERR_MSG, databean.getmErrMsg());
            dataMap.put(Constants.VTWRES_V_RESULT_CODE, databean.getvResultCode());

            // 個人情報取得
            String value = null;
            for(int i=0; i<PRIVATEINFO.length; i++){
                value = request.getParameter(PRIVATEINFO[i]);
                if(!StringUtil.isNullStr(value)){
                    dataMap.put(PRIVATEINFO[i], value);
                }
            }

            dataBox.setBox(orderId, dataMap);
            response.setContentType(CONTENT_TYPE);
            ServletOutputStream out = response.getOutputStream();

            out.println("OK");
        } else {
            log.info("内容確認失敗");
            response.setContentType(CONTENT_TYPE);
            ServletOutputStream out = response.getOutputStream();

            out.println("ERR");
        }
    }

    /**
     * GETされたリクエストを受け取る
     *
     * @param request リクエスト
     * @param response レスポンス
     * @throws IOException 入出力例外
     * @throws ServletException サーブレットの例外
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        doPost(request, response);
    }

}
