package jp.co.veritrans.vtweb.sample.server.action;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;

import jp.co.veritrans.vtweb.mdk.bean.CommodityDetail;
import jp.co.veritrans.vtweb.mdk.bean.EncryptionKey;
import jp.co.veritrans.vtweb.mdk.common.Constants;
import jp.co.veritrans.vtweb.mdk.common.DateUtil;
import jp.co.veritrans.vtweb.mdk.common.HashCodeCreater4Merchant;
import jp.co.veritrans.vtweb.mdk.common.MDKUtils;
import jp.co.veritrans.vtweb.mdk.common.StringUtil;
import jp.co.veritrans.vtweb.mdk.conf.MerchantConf;

import org.apache.log4j.Logger;

/**
 * 暗号鍵取得を行うActionクラス
 */
public class ConfirmAction extends MerchantSampleAction {

    /**
     * ログ出力用インスタンス
     */
    private static Logger log = Logger.getLogger(ConfirmAction.class);

    /**
     * POSTで送信
     *
     * @param form 情報を取得するアクションフォームクラス
     * @param commodityDetail 商品情報の配列
     * @return ブラウザ側の暗号鍵
     * @throws NoSuchAlgorithmException
     */
    @Override
    protected EncryptionKey send(final String sessionId, final String orderId, final CommodityForm form,
            final CommodityDetail[] commodityDetail) throws NoSuchAlgorithmException {
        log.info("POST送信開始");
        try {
            // VTWEBはクライアント証明書を配布しない
            HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
                @Override
                public boolean verify(final String host, final SSLSession ses) {
                    return true;
                }
            });

            // URLクラスのインスタンスを生成
            MerchantConf info = MerchantConf.getInfo();

            // TLS1.2通信用にSSLContextを設定
            SSLContext sc = SSLContext.getInstance("TLSv1.2");
            sc.init(null, null, new java.security.SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());

            URL accessURL = new URL(info.getVtWebRegistUrl());

            // 接続します
            URLConnection con = accessURL.openConnection();
            con.setRequestProperty("Content-Type", "application/x-www-form-urlencoded; charset=" + Constants.VTW_ENCODE);
            con.setRequestProperty("Accept-Language", "ja");
            con.setRequestProperty("Connection", "close");
            con.setDoOutput(true);

            // 出力ストリームを取得
            BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(con.getOutputStream(), Constants.VTW_ENCODE));
            Map map = createParameterMap(sessionId, orderId, form, commodityDetail);
            String postData = getPostData(map);
            log.info("送信データ:" + postData);

            bw.write(postData);
            bw.flush();
            bw.close();

            // 入力ストリームを取得
            BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream(), Constants.VTW_ENCODE));

            // 一行ずつ読み込みます
            EncryptionKey ret = new EncryptionKey();
            String line;
            try {
                while ((line = in.readLine()) != null) {
                    // 表示します
                    log.info("返信内容：" + line);
                    if (line.startsWith("BROWSER_ENCRYPTION_KEY")) {
                        String tmp = line.substring(23);
                        ret.setBrowserEncryptionKey(tmp);
                    } else if (line.startsWith("ERROR_MESSAGE")) {
                        String tmp = URLDecoder.decode(line.substring(14), Constants.VTW_ENCODE);
                        ret.setErrorMessage(tmp);
                    }
                }
            } catch (Throwable th) {
                log.error("受信結果解析エラーです", th);
                return null;
            }
            // 入力ストリームを閉じます
            in.close();

            log.info("POST通信完了");

            return ret;

        } catch (MalformedURLException e) {
            log.error("接続エラーです。", e);
            return null;
        } catch (IOException e) {
            log.error("接続エラーです。", e);
            return null;
        } catch (KeyManagementException e) {
            log.error("接続エラーです。", e);
            return null;
        }
    }

    /**
     * POST用の商品情報を取得する
     *
     * @param commodity 商品情報の配列
     * @return POST用の商品情報
     */
    private String[][] getCommodityDetail(final CommodityDetail[] commodity) {
        String[][] info = new String[6][commodity.length];
        for (int i = 0; i < commodity.length; i++) {
            info[0][i] = commodity[i].getCommodityId();
            info[2][i] = commodity[i].getJanCode();
            info[3][i] = commodity[i].getCommodityName();
            info[4][i] = commodity[i].getCommodityUnit().toString();
            info[5][i] = commodity[i].getCommodityNum().toString();
        }
        return info;
    }

    /**
     * POST用に整形したデータを返却する。
     *
     * @param map 情報が登録されたマップ
     * @return POST用データ
     * @throws UnsupportedEncodingException
     */
    private String getPostData(final Map map) throws UnsupportedEncodingException {

        StringBuffer buff = new StringBuffer();

        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            String keyName = (String)it.next();
            Object obj = map.get(keyName);
            if (obj != null) {
                String appendStr = null;
                if (obj instanceof Calendar) {
                    DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd HH:mm:ss");
                    appendStr = dateFormat.format(((Calendar)obj).getTime());
                    buff.append(keyName + "=" + URLEncoder.encode(appendStr, Constants.VTW_ENCODE));
                } else if (obj instanceof String[]) {
                    String[] strs = (String[])obj;
                    for (int i = 0; i < strs.length; i++) {
                        buff.append(keyName + "=" + URLEncoder.encode(strs[i], Constants.VTW_ENCODE));
                        if (i < strs.length - 1) {
                            buff.append("&");
                        }
                    }
                } else {
                    appendStr = obj.toString();
                    buff.append(keyName + "=" + URLEncoder.encode(appendStr, Constants.VTW_ENCODE));
                }
            }
            if (it.hasNext()) {
                buff.append("&");
            }
        }

        log.info("送信内容：" + buff.toString());
        return buff.toString();
    }

    /**
     * パラメータを決定しマップへ登録
     *
     * @param form 情報を取得するアクションフォームクラス
     * @param commodityDetail 商品情報の配列
     * @return 情報が登録されたマップ
     * @throws NoSuchAlgorithmException
     * @throws UnsupportedEncodingException
     */
    private Map createParameterMap(final String sessionId, final String orderId, final CommodityForm form,
            final CommodityDetail[] commodityDetail) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        Map settlementInfo = new LinkedHashMap();
        MerchantConf merchantInfo = MerchantConf.getInfo();

        // 決済方法
        String type = null;
        if (StringUtil.isNullStr(form.getSettlementType())) {
            type = merchantInfo.getSettlementType(); // 決済情報入力時にVTWEBMDK.propertiesで定義したデフォルト値をセット
            form.setSettlementType(type);
        } else {
            type = form.getSettlementType();
        }
        settlementInfo.put("SETTLEMENT_TYPE", type);

        // 決済方法サブタイプ
        settlementInfo.put("SETTLEMENT_SUB_TYPE", form.getSettlementSubType());
        // マーチャントID
        settlementInfo.put("MERCHANT_ID", merchantInfo.getMerchantID());
        // 取引ID
        settlementInfo.put("ORDER_ID", orderId);
        // SessionID
        settlementInfo.put("SESSION_ID", sessionId);
        // 購入金額
        int amount = Integer.parseInt(form.getAmount());
        int shippingAmount = 0;
        int totalAmount = 0;
        if (form.getShippingAmount() != null) {
            shippingAmount = Integer.parseInt(form.getShippingAmount());
            // 送料合計
            settlementInfo.put("SHIPPING_AMOUNT", form.getShippingAmount());
        }
        totalAmount = amount + shippingAmount;
        settlementInfo.put("AMOUNT", String.valueOf(totalAmount));
        // カード売上フラグ
        settlementInfo.put("CARD_CAPTURE_FLAG", form.getCardCaptureFlag());
        // モバイルEdy用ショップ名
        settlementInfo.put("SHOP_NAME", form.getShopName());
        // Suica用商品名
        settlementInfo.put("SCREEN_TITLE", form.getScreenTitle());
        // 請求内容
        settlementInfo.put("CONTENTS", form.getContents());
        // 請求内容カナ
        settlementInfo.put("CONTENTS_KANA", form.getContentsKana());
        // 支払取消期限
        settlementInfo.put("TIMELIMIT_OF_CANCEL", form.getTimelimitOfCancel());
        // 注文番号
        settlementInfo.put("ORICO_ORDER_NO", MDKUtils.createDummyReceiptNo());
        // 商品合計金額
        settlementInfo.put("COMMODITY_AMOUNT", form.getAmount());
        // 配送先郵便番号
        String shippingzip = "";
        if (!StringUtil.isNullStr(form.getShippingZipCode1())) {
            if (!StringUtil.isNullStr(form.getShippingZipCode2())) {
                shippingzip = StringUtil.emptyString(form.getShippingZipCode1()) + "-"
                        + StringUtil.emptyString(form.getShippingZipCode2());
            } else {
                shippingzip = StringUtil.emptyString(form.getShippingZipCode1());
            }
        } else if (!StringUtil.isNullStr(form.getShippingZipCode2())) {
            shippingzip = StringUtil.emptyString(form.getShippingZipCode2());
        } else {
            shippingzip = "123-4567";
        }
        settlementInfo.put("SHIPPING_ZIP_CODE", shippingzip);

        // マーチャント生成ハッシュ値
        HashCodeCreater4Merchant hash = new HashCodeCreater4Merchant();
        String hashCode = hash.getHash(merchantInfo.getMerchantID(), type, orderId, String.valueOf(totalAmount));
        settlementInfo.put("MERCHANTHASH", hashCode);

        // 決済完了後戻りURL
        settlementInfo.put("FINISH_PAYMENT_RETURN_URL", merchantInfo.getFinishPaymentReturnUrl());
        // 未決済時戻りURL
        settlementInfo.put("UNFINISH_PAYMENT_RETURN_URL", merchantInfo.getUnfinishPaymentReturnUrl());
        // 決済エラー時戻りURL
        settlementInfo.put("ERROR_PAYMENT_RETURN_URL", merchantInfo.getErrorPaymentReturnUrl());
        // 決済結果通知先URL
        settlementInfo.put("FINISH_PAYMENT_ACCESS_URL", merchantInfo.getFinishPaymentAccessUrl());
        // 言語選択可否フラグ
        settlementInfo.put("LANG_ENABLE_FLAG", merchantInfo.getLangEnableFlag());
        // 使用言語
        settlementInfo.put("LANG", merchantInfo.getLang());
        // ダミー決済フラグ
        settlementInfo.put("DUMMY_PAYMENT_FLAG", merchantInfo.getDummyPaymentFlag());

        // 商品情報
        if (commodityDetail != null) {
            String[][] info = getCommodityDetail(commodityDetail);
            // 商品ID
            settlementInfo.put("COMMODITY_ID", info[0]);
            // 単価
            settlementInfo.put("COMMODITY_UNIT", info[4]);
            // 個数
            settlementInfo.put("COMMODITY_NUM", info[5]);
            // 商品名
            settlementInfo.put("COMMODITY_NAME", info[3]);
            // JANコード
            settlementInfo.put("JAN_CODE", info[2]);
        }

        // 登録時振込番号
        settlementInfo.put("ENTRY_TRANSFER_NUMBER", form.getEntryTransferNumber());
        // 振込期限
        settlementInfo.put("TRANSFER_EXPIRED_DATE", form.getTransferExpiredDate());
        // 口座管理方式
        settlementInfo.put("ACCOUNT_MANAGE_TYPE", form.getAccountManageType());

        // 支店コード
        settlementInfo.put("BRANCH_CODE", form.getBranchCode());
        // 口座番号
        settlementInfo.put("ACCOUNT_NUMBER", form.getAccountNumber());
        // 銀行口座関連付け拒否フラグ
        if (form.isRejectBankAccount()) {
            settlementInfo.put("REJECT_BANK_ACCOUNT", "1");
        } else {
            settlementInfo.put("REJECT_BANK_ACCOUNT", "0");
        }
        // PayNowID使用フラグ
        if (form.isPaynowidFlag()) {
            settlementInfo.put("PAYNOWID_FLAG", "1");
            settlementInfo.put("ACCOUNT_ID", form.getAccountId());
        } else {
            settlementInfo.put("PAYNOWID_FLAG", "0");
        }

        return settlementInfo;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * jp.co.veritrans.sample.server.action.MerchantSampleAction#addInfomation
     * (jp.co.veritrans.sample.server.action.CommodityForm)
     */
    @Override
    protected void addInfomation(final CommodityForm form) {
        String type = form.getSettlementType();
        if (StringUtil.isNullStr(type) || "00".equals(type)) {
            MerchantConf info = MerchantConf.getInfo();
            // 支払い方法指定無しの場合は、全支払方法に対応できるよう、固有項目もセット

            // 支払取消期限
            int cancelLimit = Constants.DEFAULT_CANCEL_LIMIT_DAYS;
            form.setTimelimitOfCancel(DateUtil.getDateYYYYMMDD(cancelLimit));
            // 売り上げフラグ
            if ("1".equals(info.getCardCaptureFlag())) {
                form.setCardCaptureFlag("1");
            } else {
                form.setCardCaptureFlag("0");
            }

            // Suica用画面タイトル
            String tmp = info.getScreenTitle();
            if (!StringUtil.isNullStr(tmp)) {
                form.setScreenTitle(tmp);
            }

            // ショップ名
            tmp = info.getShopName();
            if (!StringUtil.isNullStr(tmp)) {
                form.setShopName(tmp);
            }

            // 請求内容
            if (StringUtil.isNullStr(form.getContents())) {
                form.setContents(info.getContents());
            }

            // 請求内容カナ
            if (StringUtil.isNullStr(form.getContentsKana())) {
                form.setContentsKana(info.getContentsKana());
            }
        }
    }
}
