package jp.co.veritrans.vtweb.sample.server.action.bank;

import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;

import jp.co.veritrans.vtweb.mdk.common.Constants;
import jp.co.veritrans.vtweb.mdk.common.StringUtil;
import jp.co.veritrans.vtweb.sample.server.action.CommodityForm;

/**
 * 銀行決済のActionFormクラス
 */
public class BankForm extends CommodityForm {

    /**
     * serial version UID.
     */
    private static final long serialVersionUID = 2149869999536108225L;

    /**
     * ログ出力用インスタンス
     */
    private static Logger log = Logger.getLogger(BankForm.class);

    /**
     * 有効な決済方法サブタイプを保持するArrayList
     */
    private static ArrayList subtype = new ArrayList();
    static{
        subtype.add(Constants.BANK_SUBTYPE_ATM);
        subtype.add(Constants.BANK_SUBTYPE_NETBANK);
    }
    
    /**
     * コンストラクタ
     */
    public BankForm() {
        super();
        setSettlementType(Constants.SETTLEMENT_TYPE_BANK);
    }

    /**
     * エラーチェック。厳密なチェックはVTWebで行うため、ここでは形式的なチェックのみでも問題ありませんが、
     * レスポンスタイム等を考慮し、必要に応じてチェック内容をご検討下さい。
     * 
     * @param mapping このインスタンスを選択するために使用するマッピング
     * @param request 処理中のHTTPリクエスト
     * @return エラーメッセージ
     */
    public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
        ActionErrors errors = new ActionErrors();

        log.info("銀行決済の決済情報をチェックします");
        // 決済方法サブタイプ
        if(!StringUtil.isNullStr(getSettlementSubType()) && !subtype.contains(getSettlementSubType())){
            addError(errors, "error.bank.subtype.invalid");    
        }
    
        super.validate(mapping, request, errors);
        return errors;
    }
}
